// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Scene.hpp"

using namespace Dawn;

Scene::Scene(
  const std::weak_ptr<Game> game,
  const std::function<void(Scene&)> sceneInitializer
) :
  game(game),
  sceneInitializer(sceneInitializer)
{
}

void Scene::stage() {
  Scene &selfReference = *this;
  sceneInitializer(selfReference);
}

void Scene::update() {
  // Initialize new scene items
  if(!hasInitialized) {
    hasInitialized = true;
    for(auto &item : sceneItems) {
      item->init();
    }
  }
  
  // Remove stale scene items.
  auto itRemove = sceneItemsToRemove.begin();
  while(itRemove != sceneItemsToRemove.end()) {
    auto item = *itRemove;
    auto it = std::find(sceneItems.begin(), sceneItems.end(), item);
    if(it != sceneItems.end()) sceneItems.erase(it);
    itRemove++;
  }
  sceneItemsToRemove.clear();

  // Tick scene items.
  float_t delta = getGame()->timeManager.delta;
  if(paused) {
    onPausedUpdate.emit(delta);
  } else {
    onUnpausedUpdate.emit(delta);
    
    onTimeout.tick(delta);
    onInterval.tick(delta);
  }
}

std::shared_ptr<Game> Scene::getGame() {
  return game.lock();
}

std::shared_ptr<SceneItem> Scene::createSceneItem() {
  auto item = std::make_shared<SceneItem>(shared_from_this());
  sceneItems.push_back(item);
  return item;
}

void Scene::removeItem(const std::shared_ptr<SceneItem> item) {
  sceneItemsToRemove.push_back(item);
}

Scene::~Scene() {
}