// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"
#include "Card.hpp"

#define POKER_WINNING_CONFIDENCE_ROYAL_FLUSH 1.0f
#define POKER_WINNING_CONFIDENCE_STRAIGHT_FLUSH 0.99f
#define POKER_WINNING_CONFIDENCE_FOUR_OF_A_KIND 0.9f
#define POKER_WINNING_CONFIDENCE_FULL_HOUSE 0.85f
#define POKER_WINNING_CONFIDENCE_FLUSH 0.8f
#define POKER_WINNING_CONFIDENCE_STRAIGHT 0.7f
#define POKER_WINNING_CONFIDENCE_THREE_OF_A_KIND 0.5f
#define POKER_WINNING_CONFIDENCE_TWO_PAIR 0.4f
#define POKER_WINNING_CONFIDENCE_PAIR 0.2f
#define POKER_WINNING_CONFIDENCE_HIGH_CARD 0.1f

/** How many cards in the winning set */
#define POKER_WINNING_SET_SIZE 5

namespace Dawn {
  class PokerPlayer;

  enum PokerWinningType {
    POKER_WINNING_TYPE_NULL,
    POKER_WINNING_TYPE_ROYAL_FLUSH,
    POKER_WINNING_TYPE_STRAIGHT_FLUSH,
    POKER_WINNING_TYPE_FOUR_OF_A_KIND,
    POKER_WINNING_TYPE_FULL_HOUSE,
    POKER_WINNING_TYPE_FLUSH,
    POKER_WINNING_TYPE_STRAIGHT,
    POKER_WINNING_TYPE_THREE_OF_A_KIND,
    POKER_WINNING_TYPE_TWO_PAIR,
    POKER_WINNING_TYPE_PAIR,
    POKER_WINNING_TYPE_HIGH_CARD
  };

  struct PokerWinning {
    public:
      /**
       * Get the confidence of the bet for a given winning type.
       * 
       * @param type Winning type type.
       * @return The confidence.
       */
      static float_t getWinningTypeConfidence(enum PokerWinningType type);

      /**
       * Compares two winning sets. The returned card is the kicker if the LEFT
       * side is the winner. If LEFT is not a winner then 0xFF will be returned.
       * 
       * @param left Left winning set.
       * @param right Right winning set.
       * @return The kicker card from left's hand or 0xFF if not the winner.
       */
      static struct Card compare(
        struct PokerWinning *left,
        struct PokerWinning *right
      );

      /** Winning Type */
      enum PokerWinningType type;
      /** The full set of both the dealer and player's hand */
      std::vector<struct Card> full;
      /** Holds the winning set */
      std::vector<struct Card> set;
      /** If there was a kicker card it will be here */
      struct Card kicker;
      /* The player this winning state belongs to */
      PokerPlayer *player;
      
      PokerWinning() : kicker(0xFF) {}

      /**
       * Fills the remaining cards for a given poker player winning hand.
       * Essentially this will just take the highest cards and slot them into
       * the array. This also sorts the cards.
       * 
       * @param winning Pointer to the poker winning to fill out.
       */
      void fillRemaining();
  };
}